#include <stdint.h>
#include "lcd.h"
#include <unistd.h>

void LCD_WriteCmd (uint8_t cmd) //write commant to RA8875
{
	*((unsigned *)(io_reg0 )) |= MASK_RD_W;
	*((unsigned *)(io_reg0 )) |= MASK_RS_W;
	
	*((unsigned *)(io_reg0 )) &= ((~MASK_DATA_W) | cmd ); //write cmd

	*((unsigned *)(io_reg0 )) &= (~ MASK_WR_W);
	*((unsigned *)(io_reg0 )) |= MASK_WR_W;

	*((unsigned *)(io_reg0 )) |= MASK_DATA_W;  //set data to 0xfffffff

	*((unsigned *)(io_reg0 )) &= (~ MASK_RS_W);

	return;
}

void LCD_WriteData (uint16_t data)  //write data to RA8875
{

	*((unsigned *)(io_reg0 )) |= MASK_RD_W;
	//*((unsigned *)(io_reg0 )) &= (~ MASK_CS_W);
	*((unsigned *)(io_reg0 )) &= (~ MASK_RS_W);
	
	*((unsigned *)(io_reg0 )) &= ((~MASK_DATA_W) | data ); //write data

	*((unsigned *)(io_reg0 )) &= (~ MASK_WR_W);
	*((unsigned *)(io_reg0 )) |= MASK_WR_W;
	//*((unsigned *)(io_reg0 )) |= MASK_CS_W;

	*((unsigned *)(io_reg0 )) |= MASK_DATA_W;  //set data to 0xfffffff

	return;
}

void Write_Dir(uint8_t Cmd,uint16_t Data)
{
  	LCD_WriteCmd(Cmd);
  	LCD_WriteData(Data);

    return;
}

/////////////////Set the working window area
void Active_Window(uint16_t XL,uint16_t XR ,uint16_t YT ,uint16_t YB)
{
	uint16_t temp;
    //setting active window X
	temp=XL;   
    LCD_WriteCmd(0x30);//HSAW0
	LCD_WriteData(temp);
	temp=XL>>8;   
    LCD_WriteCmd(0x31);//HSAW1	   
	LCD_WriteData(temp);

	temp=XR;   
    LCD_WriteCmd(0x34);//HEAW0
	LCD_WriteData(temp);
	temp=XR>>8;   
    LCD_WriteCmd(0x35);//HEAW1	   
	LCD_WriteData(temp);

    //setting active window Y
	temp=YT;   
    LCD_WriteCmd(0x32);//VSAW0
	LCD_WriteData(temp);
	temp=YT>>8;   
    LCD_WriteCmd(0x33);//VSAW1	   
	LCD_WriteData(temp);

	temp=YB;   
    LCD_WriteCmd(0x36);//VEAW0
	LCD_WriteData(temp);
	temp=YB>>8;   
    LCD_WriteCmd(0x37);//VEAW1	   
	LCD_WriteData(temp);

	return;
}


/////////////PLL setting
void LCD_PLL_Ini(void)
{
    LCD_WriteCmd(0x88);      
    LCD_WriteData(0x0C);
    usleep(1000);    
    LCD_WriteCmd(0x89);        
    LCD_WriteData(0x02);  
   	usleep(1000);
   	return;
}	

void LCD_Init(void)
{ 	
    LCD_PLL_Ini();
	LCD_WriteCmd(0x10);	 //SYSR   bit[4:3] color  bit[2:1]=  MPU interface
	LCD_WriteData(0x0f);   //       16 BIT     65K						 
	
	LCD_WriteCmd(0x04);    //PCLK
	LCD_WriteData(0x81);   //
	usleep(1000);

	 //Horizontal set
	LCD_WriteCmd(0x14); //HDWR//Horizontal Display Width Setting Bit[6:0]                      
	LCD_WriteData(0x63);//Horizontal display width(pixels) = (HDWR + 1)*8                      
	LCD_WriteCmd(0x15);//Horizontal Non-Display Period Fine Tuning Option Register (HNDFTR)   
	LCD_WriteData(0x00);//Horizontal Non-Display Period Fine Tuning(HNDFT) [3:0]               
	LCD_WriteCmd(0x16); //HNDR//Horizontal Non-Display Period Bit[4:0]                         
	LCD_WriteData(0x03);//Horizontal Non-Display Period (pixels) = (HNDR + 1)*8                
	LCD_WriteCmd(0x17); //HSTR//HSYNC Start Position[4:0]                                      
	LCD_WriteData(0x03);//HSYNC Start Position(PCLK) = (HSTR + 1)*8                            
	LCD_WriteCmd(0x18); //HPWR//HSYNC Polarity ,The period width of HSYNC.                     
	LCD_WriteData(0x0B);//HSYNC Width [4:0]   HSYNC Pulse width(PCLK) = (HPWR + 1)*8    
	 //Vertical set                             
	LCD_WriteCmd(0x19); //VDHR0 //Vertical Display Height Bit [7:0]                            
	LCD_WriteData(0xdf);//Vertical pixels = VDHR + 1                                           
	LCD_WriteCmd(0x1a); //VDHR1 //Vertical Display Height Bit [8]                              
	LCD_WriteData(0x01);//Vertical pixels = VDHR + 1                                           
	LCD_WriteCmd(0x1b); //VNDR0 //Vertical Non-Display Period Bit [7:0]                        
	LCD_WriteData(0x20);//Vertical Non-Display area = (VNDR + 1)                               
	LCD_WriteCmd(0x1c); //VNDR1 //Vertical Non-Display Period Bit [8]                          
	LCD_WriteData(0x00);//Vertical Non-Display area = (VNDR + 1)                               
	LCD_WriteCmd(0x1d); //VSTR0 //VSYNC Start Position[7:0]                                    
	LCD_WriteData(0x16);//VSYNC Start Position(PCLK) = (VSTR + 1)                              
	LCD_WriteCmd(0x1e); //VSTR1 //VSYNC Start Position[8]                                      
	LCD_WriteData(0x00);//VSYNC Start Position(PCLK) = (VSTR + 1)                              
	LCD_WriteCmd(0x1f); //VPWR //VSYNC Polarity ,VSYNC Pulse Width[6:0]                        
	LCD_WriteData(0x01);//VSYNC Pulse Width(PCLK) = (VPWR + 1)   
   

	Active_Window(0,799,0,479);

	LCD_WriteCmd(0x8a);//PWM setting
	LCD_WriteData(0x80);
	LCD_WriteCmd(0x8a);//PWM setting
	LCD_WriteData(0x81);//open PWM
	LCD_WriteCmd(0x8b);//Backlight brightness setting
	LCD_WriteData(0xff);//Brightness parameter 0xff-0x00

		 	////////////RA8875 internal input character test
    //Text_Foreground_Color1(color_black);//Set the foreground color
	Text_Background_Color1(color_black);//Set the background color		
	Active_Window(0,799,0,479);;//Set the work window size
	Write_Dir(0X8E,0X80);//Start screen clearing (display window)
    usleep(10000);
    Write_Dir(0x2E,0x01);//Set the characters mode 16 x16 / spacing 1
    Write_Dir(0x21,0x10);//Select the internal CGROM  ISO/IEC 8859-1.

	//display on
	LCD_WriteCmd(0x01);
	LCD_WriteData(0x80);

	return;
}

////////////////Foreground color settings
void Text_Foreground_Color1(uint16_t b_color)
{
	
	LCD_WriteCmd(0x63);//BGCR0
	LCD_WriteData((uint16_t)(b_color>>11));
	
	LCD_WriteCmd(0x64);//BGCR0
	LCD_WriteData((uint16_t)(b_color>>5));
	
	LCD_WriteCmd(0x65);//BGCR0
	LCD_WriteData((uint16_t)(b_color));

	return;
} 

///////////////Background color settings
void Text_Background_Color1(uint16_t b_color)
{
	
	LCD_WriteCmd(0x60);//BGCR0
	LCD_WriteData((uint16_t)(b_color>>11));
	
	LCD_WriteCmd(0x61);//BGCR0
	LCD_WriteData((uint16_t)(b_color>>5));
	
	LCD_WriteCmd(0x62);//BGCR0
	LCD_WriteData((uint16_t)(b_color));

	return;
} 

void MemoryWrite_Position(uint16_t X,uint16_t Y)
{
	uint16_t temp;

	temp=X;   
    LCD_WriteCmd(0x46);
	LCD_WriteData(temp);
	temp=X>>8;   
    LCD_WriteCmd(0x47);	   
	LCD_WriteData(temp);

	temp=Y;   
    LCD_WriteCmd(0x48);
	LCD_WriteData(temp);
	temp=Y>>8;   
    LCD_WriteCmd(0x49);	   
	LCD_WriteData(temp);

	return;
}

void LCD_PutPixel(uint16_t x, uint16_t y, uint16_t color)
{
	MemoryWrite_Position(x,y);
	LCD_WriteCmd(0x02);
	LCD_WriteData(color);

	return;
}

////////////////Text write position
void FontWrite_Position(uint16_t X,uint16_t Y)
{
	uint16_t temp;
	temp=X;   
    LCD_WriteCmd(0x2A);
	LCD_WriteData(temp);
	temp=X>>8;   
    LCD_WriteCmd(0x2B);	   
	LCD_WriteData(temp);

	temp=Y;   
    LCD_WriteCmd(0x2C);
	LCD_WriteData(temp);
	temp=Y>>8;   
    LCD_WriteCmd(0x2D);	   
	LCD_WriteData(temp);

	return;
}

//////////////writing text
void String(uint8_t *str)
{   
    Write_Dir(0x40,0x80);//Set the character mode
	LCD_WriteCmd(0x02);
	while(*str != '\0')
	{
	 LCD_WriteData(*str);
	 ++str;	 	
	 //usleep(1);
	 while(!((*((unsigned *)(io_reg0 +2))) & MASK_WINT_R));		
	} 
}