----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 13.01.2018 17:33:39
-- Design Name: 
-- Module Name: lcd - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd is
    Port ( DIN          : in STD_LOGIC_VECTOR (15 downto 0);
           DOUT         : out STD_LOGIC_VECTOR (15 downto 0);
           NEW_DAT      : in STD_LOGIC;
           R_W          : in STD_LOGIC;
           D_I          : in STD_LOGIC;
           VALID        : out STD_LOGIC;
           INT          : out STD_LOGIC;
           LWAIT        : out STD_LOGIC;
           CS           : in STD_LOGIC;
           BLCONT       : in STD_LOGIC;
           BSY          : out STD_LOGIC;
           
           LCD_DAT_O    : out STD_LOGIC_VECTOR (15 downto 0);
           LCD_DAT_i    : in STD_LOGIC_VECTOR (15 downto 0);
           LCD_DAT_T    : out STD_LOGIC_VECTOR (15 downto 0);
           LCD_RD       : out STD_LOGIC;
           LCD_WR       : out STD_LOGIC;
           LCD_RS       : out STD_LOGIC;
           LCD_BLCONT   : out STD_LOGIC;
           LCD_INT      : in STD_LOGIC;
           LCD_WAIT     : in STD_LOGIC;
           LCD_CS       : out STD_LOGIC;
           
           CLK_EN       : in STD_LOGIC;
           CLK          : in STD_LOGIC);
end lcd;

architecture Behavioral of lcd is
    TYPE t_state IS (st_default, st_read, st_wr_dat1, st_wr_dat2, st_wr_inst1, st_wr_inst2, st_rd_dat1, st_rd_dat2, st_rd_inst1, st_rd_inst2);
    SIGNAL pres_st : t_state := st_default;
    SIGNAL next_st : t_state;
    SIGNAL new_dat_prev : STD_LOGIC := '0';
    SIGNAL lcd_dat_o_buf : STD_LOGIC_VECTOR (15 downto 0) := (others => '1');
    SIGNAL lcd_dat_t_buf : STD_LOGIC_VECTOR (15 downto 0) := (others => '1');
    SIGNAL dout_buf : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

begin

PROCESS (pres_st, CLK_EN, NEW_DAT, new_dat_prev, R_W, D_I) BEGIN
    next_st <= pres_st;
    --new_dat_prev <= new_dat_prev;
    
    CASE pres_st IS
        WHEN st_default | st_read    =>  
                                IF CLK_EN = '1' AND NEW_DAT = NOT new_dat_prev AND R_W = '1' AND D_I = '1' THEN
                                    next_st <= st_wr_dat1;
                                    new_dat_prev <= NOT new_dat_prev;
                                ELSIF CLK_EN = '1' AND NEW_DAT = NOT new_dat_prev AND R_W = '1' AND D_I = '0' THEN
                                    next_st <= st_wr_inst1;
                                    new_dat_prev <= NOT new_dat_prev;
                                ELSIF CLK_EN = '1' AND NEW_DAT = NOT new_dat_prev AND R_W = '0' AND D_I = '1' THEN
                                    next_st <= st_rd_dat1;
                                    new_dat_prev <= NOT new_dat_prev;
                                ELSIF CLK_EN = '1' AND NEW_DAT = NOT new_dat_prev AND R_W = '0' AND D_I = '0' THEN
                                    next_st <= st_rd_inst1;
                                    new_dat_prev <= NOT new_dat_prev;
                                END IF;

        WHEN st_wr_dat1     =>  IF CLK_EN = '1' THEN next_st <= st_wr_dat2; END IF;

        WHEN st_wr_dat2     =>  IF CLK_EN = '1' THEN next_st <= st_default; END IF;

        WHEN st_wr_inst1    =>  IF CLK_EN = '1' THEN next_st <= st_wr_inst2; END IF;

        WHEN st_wr_inst2    =>  IF CLK_EN = '1' THEN next_st <= st_read; END IF;

        WHEN st_rd_dat1     =>  IF CLK_EN = '1' THEN next_st <= st_rd_dat2; END IF;

        WHEN st_rd_dat2     =>  IF CLK_EN = '1' THEN next_st <= st_read; END IF;

        WHEN st_rd_inst1    =>  IF CLK_EN = '1' THEN next_st <= st_rd_inst2; END IF;

        WHEN st_rd_inst2    =>  IF CLK_EN = '1' THEN next_st <= st_read; END IF;
    END CASE;
END PROCESS;

PROCESS (clk, pres_st) BEGIN
    IF rising_edge(clk) THEN
        BSY <= '1';
        VALID <= '0';
        lcd_dat_t_buf <= (others => '1'); -- 1 is hi Z state
        dout_buf <= dout_buf;
        lcd_dat_o_buf <= lcd_dat_o_buf;


        CASE pres_st IS
            WHEN st_default     => BSY        <= '0';
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '0';
            
            WHEN st_read        => BSY        <= '0';
                                   VALID      <= '1';
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '0';
            
            WHEN st_wr_dat1     => lcd_dat_t_buf  <= (others => '0');
                                   lcd_dat_o_buf    <= DIN; 
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '0';
                                   LCD_RS     <= '0';

            WHEN st_wr_dat2     => lcd_dat_t_buf  <= (others => '0');
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '0';

            WHEN st_wr_inst1    => lcd_dat_t_buf  <= (others => '0');
                                   lcd_dat_o_buf    <= DIN;
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '0';
                                   LCD_RS     <= '1';

            WHEN st_wr_inst2    => lcd_dat_t_buf  <= (others => '0');
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '1';

            WHEN st_rd_dat1     => LCD_RD     <= '0';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '0';

            WHEN st_rd_dat2     => dout_buf   <= LCD_DAT_i;
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '0';

            WHEN st_rd_inst1    => LCD_RD     <= '0';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '1';

            WHEN st_rd_inst2    => dout_buf   <= LCD_DAT_i;
                                   LCD_RD     <= '1';
                                   LCD_WR     <= '1';
                                   LCD_RS     <= '1';
        END CASE;
    END IF;
END PROCESS;

--stavovy registr
PROCESS (CLK) BEGIN
    IF rising_edge(CLK) THEN
        pres_st <= next_st;
        
        LCD_CS <= CS;
        LCD_BLCONT <= BLCONT;
        LWAIT <= LCD_WAIT;
        INT <= LCD_INT;
    END IF;
END PROCESS;

DOUT <= dout_buf;
LCD_DAT_O <= lcd_dat_o_buf;
LCD_DAT_O <= lcd_dat_t_buf;
end Behavioral;

